@@ -1575,7 +1575,7 @@ static ZIPARCHIVE_METHOD(open)
 	}
 
 #if LIBZIP_VERSION_MAJOR > 1 || LIBZIP_VERSION_MAJOR == 1 && LIBZIP_VERSION_MINOR >= 6
-	/* reduce BC break introduce in libzip 1.6.0
+	/* reduce BC break introduced in libzip 1.6.0
 	   "Do not accept empty files as valid zip archives any longer" */
 
 	/* open for write without option to empty the archive */
@@ -3510,6 +3510,9 @@ static PHP_MINIT_FUNCTION(zip)
 #ifdef ZIP_CM_LZMA2
 	REGISTER_ZIP_CLASS_CONST_LONG("CM_LZMA2", ZIP_CM_LZMA2);
 #endif
+#ifdef ZIP_CM_ZSTD
+	REGISTER_ZIP_CLASS_CONST_LONG("CM_ZSTD", ZIP_CM_ZSTD);
+#endif
 #ifdef ZIP_CM_XZ
 	REGISTER_ZIP_CLASS_CONST_LONG("CM_XZ", ZIP_CM_XZ);
 #endif
