@@ -52,11 +52,12 @@ static int le_zip_entry;
 	}
 /* }}} */
 
-/* {{{  PHP_ZIP_STAT_PATH(za, path, path_len, flags, sb) */
+/* {{{  PHP_ZIP_STAT_PATH(za, path, path_len, flags, sb)
+	This is always used for the first argument*/
 #define PHP_ZIP_STAT_PATH(za, path, path_len, flags, sb) \
-	if (path_len < 1) { \
-		php_error_docref(NULL, E_NOTICE, "Empty string as entry name"); \
-		RETURN_FALSE; \
+	if (path_len == 0) { \
+		zend_argument_value_error(1, "cannot be empty"); \
+		RETURN_THROWS(); \
 	} \
 	if (zip_stat(za, path, flags, &sb) != 0) { \
 		RETURN_FALSE; \
@@ -334,7 +335,7 @@ typedef struct {
 #endif
 } zip_options;
 
-static int php_zip_parse_options(zval *options, zip_options *opts)
+static int php_zip_parse_options(HashTable *options, zip_options *opts)
 /* {{{ */
 {
 	zval *option;
@@ -347,25 +348,42 @@ static int php_zip_parse_options(zval *options, zip_options *opts)
 	opts->enc_method = -1;  /* -1 to not change default */
 #endif
 
-	if ((option = zend_hash_str_find(Z_ARRVAL_P(options), "remove_all_path", sizeof("remove_all_path") - 1)) != NULL) {
+	if ((option = zend_hash_str_find(options, "remove_all_path", sizeof("remove_all_path") - 1)) != NULL) {
+		if (Z_TYPE_P(option) != IS_FALSE && Z_TYPE_P(option) != IS_TRUE) {
+			php_error_docref(NULL, E_WARNING, "Option \"remove_all_path\" must be of type bool, %s given",
+				zend_zval_type_name(option));
+		}
 		opts->remove_all_path = zval_get_long(option);
 	}
 
-	if ((option = zend_hash_str_find(Z_ARRVAL_P(options), "comp_method", sizeof("comp_method") - 1)) != NULL) {
+	if ((option = zend_hash_str_find(options, "comp_method", sizeof("comp_method") - 1)) != NULL) {
+		if (Z_TYPE_P(option) != IS_LONG) {
+			php_error_docref(NULL, E_WARNING, "Option \"comp_method\" must be of type int, %s given",
+				zend_zval_type_name(option));
+		}
 		opts->comp_method = zval_get_long(option);
 
-		if ((option = zend_hash_str_find(Z_ARRVAL_P(options), "comp_flags", sizeof("comp_flags") - 1)) != NULL) {
+		if ((option = zend_hash_str_find(options, "comp_flags", sizeof("comp_flags") - 1)) != NULL) {
+			if (Z_TYPE_P(option) != IS_LONG) {
+				php_error_docref(NULL, E_WARNING, "Option \"comp_flags\" must be of type int, %s given",
+					zend_zval_type_name(option));
+			}
 			opts->comp_flags = zval_get_long(option);
 		}
 	}
 
 #ifdef HAVE_ENCRYPTION
-	if ((option = zend_hash_str_find(Z_ARRVAL_P(options), "enc_method", sizeof("enc_method") - 1)) != NULL) {
+	if ((option = zend_hash_str_find(options, "enc_method", sizeof("enc_method") - 1)) != NULL) {
+		if (Z_TYPE_P(option) != IS_LONG) {
+			php_error_docref(NULL, E_WARNING, "Option \"enc_method\" must be of type int, %s given",
+				zend_zval_type_name(option));
+		}
 		opts->enc_method = zval_get_long(option);
 
-		if ((option = zend_hash_str_find(Z_ARRVAL_P(options), "enc_password", sizeof("enc_password") - 1)) != NULL) {
+		if ((option = zend_hash_str_find(options, "enc_password", sizeof("enc_password") - 1)) != NULL) {
 			if (Z_TYPE_P(option) != IS_STRING) {
-				php_error_docref(NULL, E_WARNING, "enc_password option expected to be a string");
+				zend_type_error("Option \"enc_password\" must be of type string, %s given",
+					zend_zval_type_name(option));
 				return -1;
 			}
 			opts->enc_password = Z_STRVAL_P(option);
@@ -373,49 +391,50 @@ static int php_zip_parse_options(zval *options, zip_options *opts)
 	}
 #endif
 
-	if ((option = zend_hash_str_find(Z_ARRVAL_P(options), "remove_path", sizeof("remove_path") - 1)) != NULL) {
+	if ((option = zend_hash_str_find(options, "remove_path", sizeof("remove_path") - 1)) != NULL) {
 		if (Z_TYPE_P(option) != IS_STRING) {
-			php_error_docref(NULL, E_WARNING, "remove_path option expected to be a string");
+			zend_type_error("Option \"remove_path\" must be of type string, %s given",
+				zend_zval_type_name(option));
 			return -1;
 		}
 
-		if (Z_STRLEN_P(option) < 1) {
-			php_error_docref(NULL, E_NOTICE, "Empty string given as remove_path option");
+		if (Z_STRLEN_P(option) == 0) {
+			zend_value_error("Option \"remove_path\" cannot be empty");
 			return -1;
 		}
 
 		if (Z_STRLEN_P(option) >= MAXPATHLEN) {
-			php_error_docref(NULL, E_WARNING, "remove_path string is too long (max: %d, %zd given)",
-						MAXPATHLEN - 1, Z_STRLEN_P(option));
+			zend_value_error("Option \"remove_path\" must be less than %d bytes", MAXPATHLEN - 1);
 			return -1;
 		}
 		opts->remove_path_len = Z_STRLEN_P(option);
 		opts->remove_path = Z_STRVAL_P(option);
 	}
 
-	if ((option = zend_hash_str_find(Z_ARRVAL_P(options), "add_path", sizeof("add_path") - 1)) != NULL) {
+	if ((option = zend_hash_str_find(options, "add_path", sizeof("add_path") - 1)) != NULL) {
 		if (Z_TYPE_P(option) != IS_STRING) {
-			php_error_docref(NULL, E_WARNING, "add_path option expected to be a string");
+			zend_type_error("Option \"add_path\" must be of type string, %s given",
+				zend_zval_type_name(option));
 			return -1;
 		}
 
-		if (Z_STRLEN_P(option) < 1) {
-			php_error_docref(NULL, E_NOTICE, "Empty string given as the add_path option");
+		if (Z_STRLEN_P(option) == 0) {
+			zend_value_error("Option \"add_path\" cannot be empty");
 			return -1;
 		}
 
 		if (Z_STRLEN_P(option) >= MAXPATHLEN) {
-			php_error_docref(NULL, E_WARNING, "add_path string too long (max: %d, %zd given)",
-						MAXPATHLEN - 1, Z_STRLEN_P(option));
+			zend_value_error("Option \"add_path\" must be less than %d bytes", MAXPATHLEN - 1);
 			return -1;
 		}
 		opts->add_path_len = Z_STRLEN_P(option);
 		opts->add_path = Z_STRVAL_P(option);
 	}
 
-	if ((option = zend_hash_str_find(Z_ARRVAL_P(options), "flags", sizeof("flags") - 1)) != NULL) {
+	if ((option = zend_hash_str_find(options, "flags", sizeof("flags") - 1)) != NULL) {
 		if (Z_TYPE_P(option) != IS_LONG) {
-			php_error_docref(NULL, E_WARNING, "flags option expected to be a integer");
+			zend_type_error("Option \"flags\" must be of type int, %s given",
+				zend_zval_type_name(option));
 			return -1;
 		}
 		opts->flags = Z_LVAL_P(option);
@@ -599,6 +618,7 @@ int php_zip_glob(char *pattern, int pattern_len, zend_long flags, zval *return_v
 	}
 
 	if ((GLOB_AVAILABLE_FLAGS & flags) != flags) {
+
 		php_error_docref(NULL, E_WARNING, "At least one of the passed flags is invalid or not supported on this platform");
 		return -1;
 	}
@@ -1101,8 +1121,7 @@ static PHP_MSHUTDOWN_FUNCTION(zip);
 static PHP_MINFO_FUNCTION(zip);
 /* }}} */
 
-/* {{{ zip_module_entry
- */
+/* {{{ zip_module_entry */
 zend_module_entry zip_module_entry = {
 	STANDARD_MODULE_HEADER,
 	"zip",
@@ -1122,8 +1141,7 @@ ZEND_GET_MODULE(zip)
 #endif
 /* set macro */
 
-/* {{{ proto resource zip_open(string filename)
-Create new zip using source uri for output */
+/* {{{ Create new zip using source uri for output */
 PHP_FUNCTION(zip_open)
 {
 	char resolved_path[MAXPATHLEN + 1];
@@ -1136,8 +1154,8 @@ PHP_FUNCTION(zip_open)
 	}
 
 	if (ZSTR_LEN(filename) == 0) {
-		php_error_docref(NULL, E_WARNING, "Empty string as source");
-		RETURN_FALSE;
+		zend_argument_value_error(1, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	if (ZIP_OPENBASEDIR_CHECKPATH(ZSTR_VAL(filename))) {
@@ -1163,8 +1181,7 @@ PHP_FUNCTION(zip_open)
 }
 /* }}} */
 
-/* {{{ proto void zip_close(resource zip)
-   Close a Zip archive */
+/* {{{ Close a Zip archive */
 PHP_FUNCTION(zip_close)
 {
 	zval * zip;
@@ -1183,8 +1200,7 @@ PHP_FUNCTION(zip_close)
 }
 /* }}} */
 
-/* {{{ proto resource zip_read(resource zip)
-   Returns the next file in the archive */
+/* {{{ Returns the next file in the archive */
 PHP_FUNCTION(zip_read)
 {
 	zval *zip_dp;
@@ -1229,8 +1245,7 @@ PHP_FUNCTION(zip_read)
 }
 /* }}} */
 
-/* {{{ proto bool zip_entry_open(resource zip_dp, resource zip_entry [, string mode])
-   Open a Zip File, pointed by the resource entry */
+/* {{{ Open a Zip File, pointed by the resource entry */
 /* Dummy function to follow the old API */
 PHP_FUNCTION(zip_entry_open)
 {
@@ -1261,8 +1276,7 @@ PHP_FUNCTION(zip_entry_open)
 }
 /* }}} */
 
-/* {{{ proto bool zip_entry_close(resource zip_ent)
-   Close a zip entry */
+/* {{{ Close a zip entry */
 PHP_FUNCTION(zip_entry_close)
 {
 	zval * zip_entry;
@@ -1276,12 +1290,12 @@ PHP_FUNCTION(zip_entry_close)
 		RETURN_THROWS();
 	}
 
-	RETURN_BOOL(SUCCESS == zend_list_close(Z_RES_P(zip_entry)));
+	zend_list_close(Z_RES_P(zip_entry));
+	RETURN_TRUE;
 }
 /* }}} */
 
-/* {{{ proto mixed zip_entry_read(resource zip_entry [, int len])
-   Read from an open directory entry */
+/* {{{ Read from an open directory entry */
 PHP_FUNCTION(zip_entry_read)
 {
 	zval * zip_entry;
@@ -1374,40 +1388,35 @@ static void php_zip_entry_get_info(INTERNAL_FUNCTION_PARAMETERS, int opt) /* {{{
 }
 /* }}} */
 
-/* {{{ proto string zip_entry_name(resource zip_entry)
-   Return the name given a ZZip entry */
+/* {{{ Return the name given a ZZip entry */
 PHP_FUNCTION(zip_entry_name)
 {
 	php_zip_entry_get_info(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
 }
 /* }}} */
 
-/* {{{ proto int zip_entry_compressedsize(resource zip_entry)
-   Return the compressed size of a ZZip entry */
+/* {{{ Return the compressed size of a ZZip entry */
 PHP_FUNCTION(zip_entry_compressedsize)
 {
 	php_zip_entry_get_info(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
 }
 /* }}} */
 
-/* {{{ proto int zip_entry_filesize(resource zip_entry)
-   Return the actual filesize of a ZZip entry */
+/* {{{ Return the actual filesize of a ZZip entry */
 PHP_FUNCTION(zip_entry_filesize)
 {
 	php_zip_entry_get_info(INTERNAL_FUNCTION_PARAM_PASSTHRU, 2);
 }
 /* }}} */
 
-/* {{{ proto string zip_entry_compressionmethod(resource zip_entry)
-   Return a string containing the compression method used on a particular entry */
+/* {{{ Return a string containing the compression method used on a particular entry */
 PHP_FUNCTION(zip_entry_compressionmethod)
 {
 	php_zip_entry_get_info(INTERNAL_FUNCTION_PARAM_PASSTHRU, 3);
 }
 /* }}} */
 
-/* {{{ proto mixed ZipArchive::open(string source [, int flags])
-Create new zip using source uri for output, return TRUE on success or the error code */
+/* {{{ Create new zip using source uri for output, return TRUE on success or the error code */
 PHP_METHOD(ZipArchive, open)
 {
 	struct zip *intern;
@@ -1426,8 +1435,8 @@ PHP_METHOD(ZipArchive, open)
 	ze_obj = Z_ZIP_P(self);
 
 	if (ZSTR_LEN(filename) == 0) {
-		php_error_docref(NULL, E_WARNING, "Empty string as source");
-		RETURN_FALSE;
+		zend_argument_value_error(1, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	if (ZIP_OPENBASEDIR_CHECKPATH(ZSTR_VAL(filename))) {
@@ -1452,16 +1461,20 @@ PHP_METHOD(ZipArchive, open)
 		ze_obj->filename = NULL;
 	}
 
-	/* reduce BC break introduce in libzip 1.6.0
-	   "Do not accept empty files as valid zip archives any longer" */
-
 	/* open for write without option to empty the archive */
+#ifdef ZIP_RDONLY
 	if ((flags & (ZIP_TRUNCATE | ZIP_RDONLY)) == 0) {
+#else
+	if ((flags & ZIP_TRUNCATE) == 0) {
+#endif
 		zend_stat_t st;
 
 		/* exists and is empty */
 		if (VCWD_STAT(resolved_path, &st) == 0 && st.st_size == 0) {
 			php_error_docref(NULL, E_DEPRECATED, "Using empty file as ZipArchive is deprecated");
+
+			/* reduce BC break introduced in libzip 1.6.0
+			   "Do not accept empty files as valid zip archives any longer" */
 			flags |= ZIP_TRUNCATE;
 		}
 	}
@@ -1478,8 +1491,7 @@ PHP_METHOD(ZipArchive, open)
 }
 /* }}} */
 
-/* {{{ proto resource ZipArchive::setPassword(string password)
-Set the password for the active archive */
+/* {{{ Set the password for the active archive */
 PHP_METHOD(ZipArchive, setPassword)
 {
 	struct zip *intern;
@@ -1506,8 +1518,7 @@ PHP_METHOD(ZipArchive, setPassword)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::close()
-close the zip archive */
+/* {{{ close the zip archive */
 PHP_METHOD(ZipArchive, close)
 {
 	struct zip *intern;
@@ -1558,8 +1569,7 @@ PHP_METHOD(ZipArchive, close)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::count()
-close the zip archive */
+/* {{{ close the zip archive */
 PHP_METHOD(ZipArchive, count)
 {
 	struct zip *intern;
@@ -1577,8 +1587,7 @@ PHP_METHOD(ZipArchive, count)
 }
 /* }}} */
 
-/* {{{ proto string ZipArchive::getStatusString()
- * Returns the status error message, system and/or zip messages */
+/* {{{ Returns the status error message, system and/or zip messages */
 PHP_METHOD(ZipArchive, getStatusString)
 {
 	zval *self = ZEND_THIS;
@@ -1621,8 +1630,7 @@ PHP_METHOD(ZipArchive, getStatusString)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::addEmptyDir(string dirname [, bool flags = 0])
-Returns the index of the entry named filename in the archive */
+/* {{{ Returns the index of the entry named filename in the archive */
 PHP_METHOD(ZipArchive, addEmptyDir)
 {
 	struct zip *intern;
@@ -1671,30 +1679,30 @@ static void php_zip_add_from_pattern(INTERNAL_FUNCTION_PARAMETERS, int type) /*
 	char *path = ".";
 	size_t  path_len = 1;
 	zend_long glob_flags = 0;
-	zval *options = NULL;
+	HashTable *options = NULL;
 	zip_options opts;
 	int found;
 	zend_string *pattern;
 
 	/* 1 == glob, 2 == pcre */
 	if (type == 1) {
-		if (zend_parse_parameters(ZEND_NUM_ARGS(), "P|la",
+		if (zend_parse_parameters(ZEND_NUM_ARGS(), "P|lh",
 					&pattern, &glob_flags, &options) == FAILURE) {
 			RETURN_THROWS();
 		}
 	} else {
-		if (zend_parse_parameters(ZEND_NUM_ARGS(), "P|sa",
+		if (zend_parse_parameters(ZEND_NUM_ARGS(), "P|sh",
 					&pattern, &path, &path_len, &options) == FAILURE) {
 			RETURN_THROWS();
 		}
 	}
 
 	if (ZSTR_LEN(pattern) == 0) {
-		php_error_docref(NULL, E_NOTICE, "Empty string as pattern");
-		RETURN_FALSE;
+		zend_argument_value_error(1, "cannot be empty");
+		RETURN_THROWS();
 	}
-	if (options && zend_hash_num_elements(Z_ARRVAL_P(options)) > 0 && (php_zip_parse_options(options, &opts) < 0)) {
-		RETURN_FALSE;
+	if (options && zend_hash_num_elements(options) > 0 && (php_zip_parse_options(options, &opts) < 0)) {
+		RETURN_THROWS();
 	}
 
 	if (type == 1) {
@@ -1778,24 +1786,21 @@ static void php_zip_add_from_pattern(INTERNAL_FUNCTION_PARAMETERS, int type) /*
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::addGlob(string pattern[,int flags [, array options]])
-Add files matching the glob pattern. See php's glob for the pattern syntax. */
+/* {{{ Add files matching the glob pattern. See php's glob for the pattern syntax. */
 PHP_METHOD(ZipArchive, addGlob)
 {
 	php_zip_add_from_pattern(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::addPattern(string pattern[, string path [, array options]])
-Add files matching the pcre pattern. See php's pcre for the pattern syntax. */
+/* {{{ Add files matching the pcre pattern. See php's pcre for the pattern syntax. */
 PHP_METHOD(ZipArchive, addPattern)
 {
 	php_zip_add_from_pattern(INTERNAL_FUNCTION_PARAM_PASSTHRU, 2);
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::addFile(string filepath[, string entryname[, int start [, int length [, int flags = FL_OVERWRITE]]]])
-Add a file in a Zip archive using its path and the name to use. */
+/* {{{ Add a file in a Zip archive using its path and the name to use. */
 PHP_METHOD(ZipArchive, addFile)
 {
 	zval *self = ZEND_THIS;
@@ -1811,8 +1816,8 @@ PHP_METHOD(ZipArchive, addFile)
 	}
 
 	if (ZSTR_LEN(filename) == 0) {
-		php_error_docref(NULL, E_NOTICE, "Empty string as filename");
-		RETURN_FALSE;
+		zend_argument_value_error(1, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	if (entry_name_len == 0) {
@@ -1829,8 +1834,7 @@ PHP_METHOD(ZipArchive, addFile)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::replaceFile(string filepath, int index[, int start [, int length [, int flags = 0]]])
-Add a file in a Zip archive using its path and the name to use. */
+/* {{{ Add a file in a Zip archive using its path and the name to use. */
 PHP_METHOD(ZipArchive, replaceFile)
 {
 	zval *self = ZEND_THIS;
@@ -1845,13 +1849,13 @@ PHP_METHOD(ZipArchive, replaceFile)
 	}
 
 	if (ZSTR_LEN(filename) == 0) {
-		php_error_docref(NULL, E_NOTICE, "Empty string as filename");
-		RETURN_FALSE;
+		zend_argument_value_error(1, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	if (index < 0) {
-		php_error_docref(NULL, E_NOTICE, "Invalid negative index");
-		RETURN_FALSE;
+		zend_argument_value_error(2, "must be greater than or equal to 0");
+		RETURN_THROWS();
 	}
 
 	if (php_zip_add_file(Z_ZIP_P(self), ZSTR_VAL(filename), ZSTR_LEN(filename),
@@ -1863,8 +1867,7 @@ PHP_METHOD(ZipArchive, replaceFile)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::addFromString(string name, string content [, int flags = FL_OVERWRITE])
-Add a file using content and the entry name */
+/* {{{ Add a file using content and the entry name */
 PHP_METHOD(ZipArchive, addFromString)
 {
 	struct zip *intern;
@@ -1913,8 +1916,7 @@ PHP_METHOD(ZipArchive, addFromString)
 }
 /* }}} */
 
-/* {{{ proto array ZipArchive::statName(string filename[, int flags])
-Returns the information about a the zip entry filename */
+/* {{{ Returns the information about a the zip entry filename */
 PHP_METHOD(ZipArchive, statName)
 {
 	struct zip *intern;
@@ -1935,8 +1937,7 @@ PHP_METHOD(ZipArchive, statName)
 }
 /* }}} */
 
-/* {{{ proto resource ZipArchive::statIndex(int index[, int flags])
-Returns the zip entry information using its index */
+/* {{{ Returns the zip entry information using its index */
 PHP_METHOD(ZipArchive, statIndex)
 {
 	struct zip *intern;
@@ -1959,8 +1960,7 @@ PHP_METHOD(ZipArchive, statIndex)
 }
 /* }}} */
 
-/* {{{ proto int ZipArchive::locateName(string filename[, int flags])
-Returns the index of the entry named filename in the archive */
+/* {{{ Returns the index of the entry named filename in the archive */
 PHP_METHOD(ZipArchive, locateName)
 {
 	struct zip *intern;
@@ -1989,8 +1989,7 @@ PHP_METHOD(ZipArchive, locateName)
 }
 /* }}} */
 
-/* {{{ proto string ZipArchive::getNameIndex(int index [, int flags])
-Returns the name of the file at position index */
+/* {{{ Returns the name of the file at position index */
 PHP_METHOD(ZipArchive, getNameIndex)
 {
 	struct zip *intern;
@@ -2015,8 +2014,7 @@ PHP_METHOD(ZipArchive, getNameIndex)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::setArchiveComment(string comment)
-Set or remove (NULL/'') the comment of the archive */
+/* {{{ Set or remove (NULL/'') the comment of the archive */
 PHP_METHOD(ZipArchive, setArchiveComment)
 {
 	struct zip *intern;
@@ -2031,8 +2029,8 @@ PHP_METHOD(ZipArchive, setArchiveComment)
 	ZIP_FROM_OBJECT(intern, self);
 
 	if (comment_len > 0xffff) {
-		php_error_docref(NULL, E_WARNING, "Comment must not exceed 65535 bytes");
-		RETURN_FALSE;
+		zend_argument_value_error(1, "must be less than 65535 bytes");
+		RETURN_THROWS();
 	}
 
 	if (zip_set_archive_comment(intern, (const char *)comment, comment_len)) {
@@ -2043,8 +2041,7 @@ PHP_METHOD(ZipArchive, setArchiveComment)
 }
 /* }}} */
 
-/* {{{ proto string ZipArchive::getArchiveComment([int flags])
-Returns the comment of an entry using its index */
+/* {{{ Returns the comment of an entry using its index */
 PHP_METHOD(ZipArchive, getArchiveComment)
 {
 	struct zip *intern;
@@ -2067,8 +2064,7 @@ PHP_METHOD(ZipArchive, getArchiveComment)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::setCommentName(string name, string comment)
-Set or remove (NULL/'') the comment of an entry using its Name */
+/* {{{ Set or remove (NULL/'') the comment of an entry using its Name */
 PHP_METHOD(ZipArchive, setCommentName)
 {
 	struct zip *intern;
@@ -2082,15 +2078,16 @@ PHP_METHOD(ZipArchive, setCommentName)
 		RETURN_THROWS();
 	}
 
-	if (name_len < 1) {
-		php_error_docref(NULL, E_NOTICE, "Empty string as entry name");
+	if (name_len == 0) {
+		zend_argument_value_error(1, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	ZIP_FROM_OBJECT(intern, self);
 
 	if (comment_len > 0xffff) {
-		php_error_docref(NULL, E_WARNING, "Comment must not exceed 65535 bytes");
-		RETURN_FALSE;
+		zend_argument_value_error(2, "must be less than 65535 bytes");
+		RETURN_THROWS();
 	}
 
 	idx = zip_name_locate(intern, name, 0);
@@ -2101,8 +2098,7 @@ PHP_METHOD(ZipArchive, setCommentName)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::setCommentIndex(int index, string comment)
-Set or remove (NULL/'') the comment of an entry using its index */
+/* {{{ Set or remove (NULL/'') the comment of an entry using its index */
 PHP_METHOD(ZipArchive, setCommentIndex)
 {
 	struct zip *intern;
@@ -2120,8 +2116,8 @@ PHP_METHOD(ZipArchive, setCommentIndex)
 	ZIP_FROM_OBJECT(intern, self);
 
 	if (comment_len > 0xffff) {
-		php_error_docref(NULL, E_WARNING, "Comment must not exceed 65535 bytes");
-		RETURN_FALSE;
+		zend_argument_value_error(2, "must be less than 65535 bytes");
+		RETURN_THROWS();
 	}
 
 	PHP_ZIP_STAT_INDEX(intern, index, 0, sb);
@@ -2132,8 +2128,7 @@ PHP_METHOD(ZipArchive, setCommentIndex)
 /* those constants/functions are only available in libzip since 0.11.2 */
 #ifdef ZIP_OPSYS_DEFAULT
 
-/* {{{ proto bool ZipArchive::setExternalAttributesName(string name, int opsys, int attr [, int flags])
-Set external attributes for file in zip, using its name */
+/* {{{ Set external attributes for file in zip, using its name */
 PHP_METHOD(ZipArchive, setExternalAttributesName)
 {
 	struct zip *intern;
@@ -2150,11 +2145,13 @@ PHP_METHOD(ZipArchive, setExternalAttributesName)
 
 	ZIP_FROM_OBJECT(intern, self);
 
-	if (name_len < 1) {
-		php_error_docref(NULL, E_NOTICE, "Empty string as entry name");
+	if (name_len == 0) {
+		zend_argument_value_error(1, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	idx = zip_name_locate(intern, name, 0);
+
 	if (idx < 0) {
 		RETURN_FALSE;
 	}
@@ -2166,8 +2163,7 @@ PHP_METHOD(ZipArchive, setExternalAttributesName)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::setExternalAttributesIndex(int index, int opsys, int attr [, int flags])
-Set external attributes for file in zip, using its index */
+/* {{{ Set external attributes for file in zip, using its index */
 PHP_METHOD(ZipArchive, setExternalAttributesIndex)
 {
 	struct zip *intern;
@@ -2191,8 +2187,7 @@ PHP_METHOD(ZipArchive, setExternalAttributesIndex)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::getExternalAttributesName(string name, int &opsys, int &attr [, int flags])
-Get external attributes for file in zip, using its name */
+/* {{{ Get external attributes for file in zip, using its name */
 PHP_METHOD(ZipArchive, getExternalAttributesName)
 {
 	struct zip *intern;
@@ -2211,11 +2206,13 @@ PHP_METHOD(ZipArchive, getExternalAttributesName)
 
 	ZIP_FROM_OBJECT(intern, self);
 
-	if (name_len < 1) {
-		php_error_docref(NULL, E_NOTICE, "Empty string as entry name");
+	if (name_len == 0) {
+		zend_argument_value_error(1, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	idx = zip_name_locate(intern, name, 0);
+
 	if (idx < 0) {
 		RETURN_FALSE;
 	}
@@ -2229,8 +2226,7 @@ PHP_METHOD(ZipArchive, getExternalAttributesName)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::getExternalAttributesIndex(int index, int &opsys, int &attr [, int flags])
-Get external attributes for file in zip, using its index */
+/* {{{ Get external attributes for file in zip, using its index */
 PHP_METHOD(ZipArchive, getExternalAttributesIndex)
 {
 	struct zip *intern;
@@ -2260,8 +2256,7 @@ PHP_METHOD(ZipArchive, getExternalAttributesIndex)
 #endif /* ifdef ZIP_OPSYS_DEFAULT */
 
 #ifdef HAVE_ENCRYPTION
-/* {{{ proto bool ZipArchive::setEncryptionName(string name, int method, [string password])
-Set encryption method for file in zip, using its name */
+/* {{{ Set encryption method for file in zip, using its name */
 PHP_METHOD(ZipArchive, setEncryptionName)
 {
 	struct zip *intern;
@@ -2278,11 +2273,13 @@ PHP_METHOD(ZipArchive, setEncryptionName)
 
 	ZIP_FROM_OBJECT(intern, self);
 
-	if (name_len < 1) {
-		php_error_docref(NULL, E_NOTICE, "Empty string as entry name");
+	if (name_len == 0) {
+		zend_argument_value_error(1, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	idx = zip_name_locate(intern, name, 0);
+
 	if (idx < 0) {
 		RETURN_FALSE;
 	}
@@ -2294,8 +2291,7 @@ PHP_METHOD(ZipArchive, setEncryptionName)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::setEncryptionIndex(int index, int method, [string password])
-Set encryption method for file in zip, using its index */
+/* {{{ Set encryption method for file in zip, using its index */
 PHP_METHOD(ZipArchive, setEncryptionIndex)
 {
 	struct zip *intern;
@@ -2319,8 +2315,7 @@ PHP_METHOD(ZipArchive, setEncryptionIndex)
 /* }}} */
 #endif
 
-/* {{{ proto string ZipArchive::getCommentName(string name[, int flags])
-Returns the comment of an entry using its name */
+/* {{{ Returns the comment of an entry using its name */
 PHP_METHOD(ZipArchive, getCommentName)
 {
 	struct zip *intern;
@@ -2339,12 +2334,13 @@ PHP_METHOD(ZipArchive, getCommentName)
 
 	ZIP_FROM_OBJECT(intern, self);
 
-	if (name_len < 1) {
-		php_error_docref(NULL, E_NOTICE, "Empty string as entry name");
-		RETURN_FALSE;
+	if (name_len == 0) {
+		zend_argument_value_error(1, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	idx = zip_name_locate(intern, name, 0);
+
 	if (idx < 0) {
 		RETURN_FALSE;
 	}
@@ -2354,8 +2350,7 @@ PHP_METHOD(ZipArchive, getCommentName)
 }
 /* }}} */
 
-/* {{{ proto string ZipArchive::getCommentIndex(int index[, int flags])
-Returns the comment of an entry using its index */
+/* {{{ Returns the comment of an entry using its index */
 PHP_METHOD(ZipArchive, getCommentIndex)
 {
 	struct zip *intern;
@@ -2378,10 +2373,9 @@ PHP_METHOD(ZipArchive, getCommentIndex)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::setCompressionName(string name, int comp_method[, int comp_flags])
-Set the compression of a file in zip, using its name */
+/* {{{ Set the compression of a file in zip, using its name */
 PHP_METHOD(ZipArchive, setCompressionName)
- {
+{
 	struct zip *intern;
 	zval *this = ZEND_THIS;
 	size_t name_len;
@@ -2396,11 +2390,13 @@ PHP_METHOD(ZipArchive, setCompressionName)
 
 	ZIP_FROM_OBJECT(intern, this);
 
-	if (name_len < 1) {
-		php_error_docref(NULL, E_NOTICE, "Empty string as entry name");
+	if (name_len == 0) {
+		zend_argument_value_error(1, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	idx = zip_name_locate(intern, name, 0);
+
 	if (idx < 0) {
 		RETURN_FALSE;
 	}
@@ -2413,8 +2409,7 @@ PHP_METHOD(ZipArchive, setCompressionName)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::setCompressionIndex(int index, int comp_method[, int comp_flags])
-Set the compression of a file in zip, using its index */
+/* {{{ Set the compression of a file in zip, using its index */
 PHP_METHOD(ZipArchive, setCompressionIndex)
 {
 	struct zip *intern;
@@ -2438,10 +2433,9 @@ PHP_METHOD(ZipArchive, setCompressionIndex)
 /* }}} */
 
 #ifdef HAVE_SET_MTIME
-/* {{{ proto bool ZipArchive::setMtimeName(string name, int timestamp[, int flags])
-Set the modification time of a file in zip, using its name */
+/* {{{ Set the modification time of a file in zip, using its name */
 PHP_METHOD(ZipArchive, setMtimeName)
- {
+{
 	struct zip *intern;
 	zval *this = ZEND_THIS;
 	size_t name_len;
@@ -2456,11 +2450,13 @@ PHP_METHOD(ZipArchive, setMtimeName)
 
 	ZIP_FROM_OBJECT(intern, this);
 
-	if (name_len < 1) {
-		php_error_docref(NULL, E_NOTICE, "Empty string as entry name");
+	if (name_len == 0) {
+		zend_argument_value_error(1, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	idx = zip_name_locate(intern, name, 0);
+
 	if (idx < 0) {
 		RETURN_FALSE;
 	}
@@ -2473,8 +2469,7 @@ PHP_METHOD(ZipArchive, setMtimeName)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::setMtimeIndex(int index, int timestamp[, int flags])
-Set the modification time of a file in zip, using its index */
+/* {{{ Set the modification time of a file in zip, using its index */
 PHP_METHOD(ZipArchive, setMtimeIndex)
 {
 	struct zip *intern;
@@ -2498,8 +2493,7 @@ PHP_METHOD(ZipArchive, setMtimeIndex)
 /* }}} */
 #endif
 
-/* {{{ proto bool ZipArchive::deleteIndex(int index)
-Delete a file using its index */
+/* {{{ Delete a file using its index */
 PHP_METHOD(ZipArchive, deleteIndex)
 {
 	struct zip *intern;
@@ -2524,8 +2518,7 @@ PHP_METHOD(ZipArchive, deleteIndex)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::deleteName(string name)
-Delete a file using its index */
+/* {{{ Delete a file using its index */
 PHP_METHOD(ZipArchive, deleteName)
 {
 	struct zip *intern;
@@ -2552,8 +2545,7 @@ PHP_METHOD(ZipArchive, deleteName)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::renameIndex(int index, string new_name)
-Rename an entry selected by its index to new_name */
+/* {{{ Rename an entry selected by its index to new_name */
 PHP_METHOD(ZipArchive, renameIndex)
 {
 	struct zip *intern;
@@ -2572,9 +2564,9 @@ PHP_METHOD(ZipArchive, renameIndex)
 
 	ZIP_FROM_OBJECT(intern, self);
 
-	if (new_name_len < 1) {
-		php_error_docref(NULL, E_NOTICE, "Empty string as new entry name");
-		RETURN_FALSE;
+	if (new_name_len == 0) {
+		zend_argument_value_error(2, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	if (zip_file_rename(intern, index, (const char *)new_name, 0) != 0) {
@@ -2585,8 +2577,7 @@ PHP_METHOD(ZipArchive, renameIndex)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::renameName(string name, string new_name)
-Rename an entry selected by its name to new_name */
+/* {{{ Rename an entry selected by its name to new_name */
 PHP_METHOD(ZipArchive, renameName)
 {
 	struct zip *intern;
@@ -2601,9 +2592,9 @@ PHP_METHOD(ZipArchive, renameName)
 
 	ZIP_FROM_OBJECT(intern, self);
 
-	if (new_name_len < 1) {
-		php_error_docref(NULL, E_NOTICE, "Empty string as new entry name");
-		RETURN_FALSE;
+	if (new_name_len == 0) {
+		zend_argument_value_error(2, "cannot be empty");
+		RETURN_THROWS();
 	}
 
 	PHP_ZIP_STAT_PATH(intern, name, name_len, 0, sb);
@@ -2616,8 +2607,7 @@ PHP_METHOD(ZipArchive, renameName)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::unchangeIndex(int index)
-Changes to the file at position index are reverted */
+/* {{{ Changes to the file at position index are reverted */
 PHP_METHOD(ZipArchive, unchangeIndex)
 {
 	struct zip *intern;
@@ -2642,8 +2632,7 @@ PHP_METHOD(ZipArchive, unchangeIndex)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::unchangeName(string name)
-Changes to the file named 'name' are reverted */
+/* {{{ Changes to the file named 'name' are reverted */
 PHP_METHOD(ZipArchive, unchangeName)
 {
 	struct zip *intern;
@@ -2672,8 +2661,7 @@ PHP_METHOD(ZipArchive, unchangeName)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::unchangeAll()
-All changes to files and global information in archive are reverted */
+/* {{{ All changes to files and global information in archive are reverted */
 PHP_METHOD(ZipArchive, unchangeAll)
 {
 	struct zip *intern;
@@ -2693,8 +2681,7 @@ PHP_METHOD(ZipArchive, unchangeAll)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::unchangeArchive()
-Revert all global changes to the archive archive.  For now, this only reverts archive comment changes. */
+/* {{{ Revert all global changes to the archive archive.  For now, this only reverts archive comment changes. */
 PHP_METHOD(ZipArchive, unchangeArchive)
 {
 	struct zip *intern;
@@ -2714,8 +2701,7 @@ PHP_METHOD(ZipArchive, unchangeArchive)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::extractTo(string pathto[, mixed files])
-Extract one or more file from a zip archive */
+/* {{{ Extract one or more file from a zip archive */
 /* TODO:
  * - allow index or array of indices
  * - replace path
@@ -2726,16 +2712,20 @@ PHP_METHOD(ZipArchive, extractTo)
 	struct zip *intern;
 
 	zval *self = ZEND_THIS;
-	zval *zval_files = NULL;
+	zend_string *files_str = NULL;
+	HashTable *files_ht = NULL;
+
 	zval *zval_file = NULL;
 	php_stream_statbuf ssb;
 	char *pathto;
 	size_t pathto_len;
 	int ret;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "p|z", &pathto, &pathto_len, &zval_files) == FAILURE) {
-		RETURN_THROWS();
-	}
+	ZEND_PARSE_PARAMETERS_START(1, 2)
+		Z_PARAM_PATH(pathto, pathto_len)
+		Z_PARAM_OPTIONAL
+		Z_PARAM_ARRAY_HT_OR_STR_OR_NULL(files_ht, files_str)
+	ZEND_PARSE_PARAMETERS_END();
 
 	ZIP_FROM_OBJECT(intern, self);
 
@@ -2750,37 +2740,29 @@ PHP_METHOD(ZipArchive, extractTo)
 		}
 	}
 
-	if (zval_files && Z_TYPE_P(zval_files) != IS_NULL) {
-		uint32_t nelems, i;
+	uint32_t nelems, i;
 
-		switch (Z_TYPE_P(zval_files)) {
-			case IS_STRING:
-				if (!php_zip_extract_file(intern, pathto, Z_STRVAL_P(zval_files), Z_STRLEN_P(zval_files))) {
-					RETURN_FALSE;
-				}
-				break;
-			case IS_ARRAY:
-				nelems = zend_hash_num_elements(Z_ARRVAL_P(zval_files));
-				if (nelems == 0 ) {
-					RETURN_FALSE;
-				}
-				for (i = 0; i < nelems; i++) {
-					if ((zval_file = zend_hash_index_find(Z_ARRVAL_P(zval_files), i)) != NULL) {
-						switch (Z_TYPE_P(zval_file)) {
-							case IS_LONG:
-								break;
-							case IS_STRING:
-								if (!php_zip_extract_file(intern, pathto, Z_STRVAL_P(zval_file), Z_STRLEN_P(zval_file))) {
-									RETURN_FALSE;
-								}
-								break;
+	if (files_str) {
+		if (!php_zip_extract_file(intern, pathto, ZSTR_VAL(files_str), ZSTR_LEN(files_str))) {
+			RETURN_FALSE;
+		}
+	} else if (files_ht) {
+		nelems = zend_hash_num_elements(files_ht);
+		if (nelems == 0 ) {
+			RETURN_FALSE;
+		}
+		for (i = 0; i < nelems; i++) {
+			if ((zval_file = zend_hash_index_find(files_ht, i)) != NULL) {
+				switch (Z_TYPE_P(zval_file)) {
+					case IS_LONG:
+						break;
+					case IS_STRING:
+						if (!php_zip_extract_file(intern, pathto, Z_STRVAL_P(zval_file), Z_STRLEN_P(zval_file))) {
+							RETURN_FALSE;
 						}
-					}
+						break;
 				}
-				break;
-			default:
-				zend_argument_type_error(2, "must be of type array|string|null, %s given", zend_zval_type_name(zval_files));
-				RETURN_THROWS();
+			}
 		}
 	} else {
 		/* Extract all files */
@@ -2798,6 +2780,7 @@ PHP_METHOD(ZipArchive, extractTo)
 			}
 		}
 	}
+
 	RETURN_TRUE;
 }
 /* }}} */
@@ -2868,24 +2851,21 @@ static void php_zip_get_from(INTERNAL_FUNCTION_PARAMETERS, int type) /* {{{ */
 }
 /* }}} */
 
-/* {{{ proto string ZipArchive::getFromName(string entryname[, int len [, int flags]])
-get the contents of an entry using its name */
+/* {{{ get the contents of an entry using its name */
 PHP_METHOD(ZipArchive, getFromName)
 {
 	php_zip_get_from(INTERNAL_FUNCTION_PARAM_PASSTHRU, 1);
 }
 /* }}} */
 
-/* {{{ proto string ZipArchive::getFromIndex(int index[, int len [, int flags]])
-get the contents of an entry using its index */
+/* {{{ get the contents of an entry using its index */
 PHP_METHOD(ZipArchive, getFromIndex)
 {
 	php_zip_get_from(INTERNAL_FUNCTION_PARAM_PASSTHRU, 0);
 }
 /* }}} */
 
-/* {{{ proto resource ZipArchive::getStream(string entryname)
-get a stream for an entry using its name */
+/* {{{ get a stream for an entry using its name */
 PHP_METHOD(ZipArchive, getStream)
 {
 	struct zip *intern;
@@ -2925,35 +2905,23 @@ static void _php_zip_progress_callback(zip_t *arch, double state, void *ptr)
 	ze_zip_object *obj = ptr;
 
 	ZVAL_DOUBLE(&cb_args[0], state);
-	if (call_user_function_ex(EG(function_table), NULL, &obj->progress_callback, &cb_retval, 1, cb_args, 0, NULL) == SUCCESS && !Z_ISUNDEF(cb_retval)) {
+	if (call_user_function(EG(function_table), NULL, &obj->progress_callback, &cb_retval, 1, cb_args) == SUCCESS && !Z_ISUNDEF(cb_retval)) {
 		zval_ptr_dtor(&cb_retval);
 	}
 }
 
-/* {{{ proto bool ZipArchive::registerProgressCallback(double rate, callable callback)
-register a progression callback: void callback(double state); */
+/* {{{ register a progression callback: void callback(double state); */
 PHP_METHOD(ZipArchive, registerProgressCallback)
 {
 	struct zip *intern;
-	zval *self = getThis();
+	zval *self = ZEND_THIS;
 	double rate;
-	zval *callback;
+	zend_fcall_info fci;
+	zend_fcall_info_cache fcc;
 	ze_zip_object *obj;
 
-	if (!self) {
-		RETURN_FALSE;
-	}
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "dz", &rate, &callback) == FAILURE) {
-		return;
-	}
-
-	/* callable? */
-	if (!zend_is_callable(callback, 0, NULL)) {
-		zend_string *callback_name = zend_get_callable_name(callback);
-		php_error_docref(NULL, E_WARNING, "Invalid callback '%s'", ZSTR_VAL(callback_name));
-		zend_string_release_ex(callback_name, 0);
-		RETURN_FALSE;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "df", &rate, &fci, &fcc) == FAILURE) {
+		RETURN_THROWS();
 	}
 
 	ZIP_FROM_OBJECT(intern, self);
@@ -2964,7 +2932,7 @@ PHP_METHOD(ZipArchive, registerProgressCallback)
 	_php_zip_progress_callback_free(obj);
 
 	/* register */
-	ZVAL_COPY(&obj->progress_callback, callback);
+	ZVAL_COPY(&obj->progress_callback, &fci.function_name);
 	if (zip_register_progress_callback_with_state(intern, rate, _php_zip_progress_callback, _php_zip_progress_callback_free, obj)) {
 		RETURN_FALSE;
 	}
@@ -2981,7 +2949,7 @@ static int _php_zip_cancel_callback(zip_t *arch, void *ptr)
 	int retval = 0;
 	ze_zip_object *obj = ptr;
 
-	if (call_user_function_ex(EG(function_table), NULL, &obj->cancel_callback, &cb_retval, 0, NULL, 0, NULL) == SUCCESS && !Z_ISUNDEF(cb_retval)) {
+	if (call_user_function(EG(function_table), NULL, &obj->cancel_callback, &cb_retval, 0, NULL) == SUCCESS && !Z_ISUNDEF(cb_retval)) {
 		retval = zval_get_long(&cb_retval);
 		zval_ptr_dtor(&cb_retval);
 	}
@@ -2989,40 +2957,27 @@ static int _php_zip_cancel_callback(zip_t *arch, void *ptr)
 	return retval;
 }
 
-/* {{{ proto bool ZipArchive::registerCancelCallback(callable callback)
-register a progression callback: int callback(double state); */
+/* {{{ register a progression callback: int callback(double state); */
 PHP_METHOD(ZipArchive, registerCancelCallback)
 {
 	struct zip *intern;
-	zval *self = getThis();
-	zval *callback;
+	zval *self = ZEND_THIS;
+	zend_fcall_info fci;
+	zend_fcall_info_cache fcc;
 	ze_zip_object *obj;
-
-	if (!self) {
-		RETURN_FALSE;
-	}
-
-	if (zend_parse_parameters(ZEND_NUM_ARGS(), "z", &callback) == FAILURE) {
-		return;
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "f", &fci, &fcc) == FAILURE) {
+		RETURN_THROWS();
 	}
 
 	ZIP_FROM_OBJECT(intern, self);
 
-	/* callable? */
-	if (!zend_is_callable(callback, 0, NULL)) {
-		zend_string *callback_name = zend_get_callable_name(callback);
-		php_error_docref(NULL, E_WARNING, "Invalid callback '%s'", ZSTR_VAL(callback_name));
-		zend_string_release_ex(callback_name, 0);
-		RETURN_FALSE;
-	}
-
 	obj = Z_ZIP_P(self);
 
 	/* free if called twice */
 	_php_zip_cancel_callback_free(obj);
 
 	/* register */
-	ZVAL_COPY(&obj->cancel_callback, callback);
+	ZVAL_COPY(&obj->cancel_callback, &fci.function_name);
 	if (zip_register_cancel_callback_with_state(intern, _php_zip_cancel_callback, _php_zip_cancel_callback_free, obj)) {
 		RETURN_FALSE;
 	}
@@ -3033,8 +2988,7 @@ PHP_METHOD(ZipArchive, registerCancelCallback)
 #endif
 
 #ifdef HAVE_METHOD_SUPPORTED
-/* {{{ proto bool ZipArchive::isCompressionMethodSupported(int method, bool enc)
-check if a compression method is available in used libzip */
+/* {{{ check if a compression method is available in used libzip */
 PHP_METHOD(ZipArchive, isCompressionMethodSupported)
 {
 	zend_long method;
@@ -3047,8 +3001,7 @@ PHP_METHOD(ZipArchive, isCompressionMethodSupported)
 }
 /* }}} */
 
-/* {{{ proto bool ZipArchive::isEncryptionMethodSupported(int method, bool enc)
-check if a encryption method is available in used libzip */
+/* {{{ check if a encryption method is available in used libzip */
 PHP_METHOD(ZipArchive, isEncryptionMethodSupported)
 {
 	zend_long method;
@@ -3136,6 +3089,9 @@ static PHP_MINIT_FUNCTION(zip)
 #ifdef ZIP_CM_LZMA2
 	REGISTER_ZIP_CLASS_CONST_LONG("CM_LZMA2", ZIP_CM_LZMA2);
 #endif
+#ifdef ZIP_CM_ZSTD
+	REGISTER_ZIP_CLASS_CONST_LONG("CM_ZSTD", ZIP_CM_ZSTD);
+#endif
 #ifdef ZIP_CM_XZ
 	REGISTER_ZIP_CLASS_CONST_LONG("CM_XZ", ZIP_CM_XZ);
 #endif
@@ -3240,8 +3196,7 @@ static PHP_MINIT_FUNCTION(zip)
 }
 /* }}} */
 
-/* {{{ PHP_MSHUTDOWN_FUNCTION
- */
+/* {{{ PHP_MSHUTDOWN_FUNCTION */
 static PHP_MSHUTDOWN_FUNCTION(zip)
 {
 	zend_hash_destroy(&zip_prop_handlers);
@@ -3250,8 +3205,7 @@ static PHP_MSHUTDOWN_FUNCTION(zip)
 }
 /* }}} */
 
-/* {{{ PHP_MINFO_FUNCTION
- */
+/* {{{ PHP_MINFO_FUNCTION */
 static PHP_MINFO_FUNCTION(zip)
 {
 	php_info_print_table_start();
